# module to ingest data to a Wendelin through HTTP endpoint
# Copyright (C) 2015  Nexedi SA and Contributors.
#                     Kirill Smelkov <kirr@nexedi.com>
#
#   This program is free software: you can Use, Study, Modify and Redistribute
#   it under the terms of the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.


require 'net/http'
require 'openssl'

# show request/response traces or not
$trace    = false


# class representing a Wendelin client
class WendelinClient

  # `streamtool_uri` - URI pointing to portal_input_data_stream "mountpoint"
  # `credentials`    # {'user' => _, 'password' => _}     TODO change to certificate
  def initialize(streamtool_uri, credentials)
    @streamtool_uri = streamtool_uri
    @credentials    = credentials
  end


  # ingest `data_chunk` to a stream referenced as `reference`
  def ingest(reference, data_chunk)
      uri = URI("#{@streamtool_uri}/ingest?reference=#{reference}")
      req = Net::HTTP::Post.new(uri)
      if @credentials.has_key?('user')
        req.basic_auth @credentials['user'], @credentials['password']
      end

      # TODO ensure content-type is 'raw', e.g. this way
      # (but then querystring ?reference=... is lost)
      # req.body = data_chunk
      # req.content_type = 'application/octet-stream'
      req.set_form_data('data_chunk' => data_chunk)

      if $trace
          puts '>>> REQUEST'
          puts "method\t=> #{req.method}"
          puts "path\t=> #{req.path}"
          puts "uri\t=> #{req.uri}"
          puts "body\t=> #{req.body}"
          puts "body_stream\t=> #{req.body_stream}"
          req.each {|h| puts "#{h}:\t#{req[h]}"}
          puts
      end

      begin
          # TODO keep connection open (so that every new ingest does not do
          # full connect again)
          res = Net::HTTP.start(uri.hostname, uri.port,
                      :use_ssl      => (uri.scheme == 'https'),
                      # NOTE = "do not check server cert"
                      # TODO move this out to conf parameters
                      :verify_mode  => OpenSSL::SSL::VERIFY_NONE
                ) do |http|
              http.request(req)
          end

      rescue
          # some http/ssl/other connection error
          puts "HTTP ERROR:"
          raise

      else
          if $trace
              puts '>>> RESPONSE'
              res.each {|h| puts "#{h}:\t#{res[h]}"}
              puts "code\t=> #{res.code}"
              puts "msg\t=> #{res.message}"
              puts "class\t=> #{res.class}"
              puts "body:", res.body
          end

          if res.kind_of?(Net::HTTPSuccess)       # res.code is 2XX
              #puts "ingested ok"
          else
              puts "FAIL:"
              res.value
          end
      end
  end

end
